#ifndef Z_HOUND_H
#define Z_HOUND_H

/*
	Hound constants
*/

#define FRAME_hattack1Sep	    122
#define FRAME_hattack1Loop	  124
#define FRAME_hattack1LoopEnd 125
#define FRAME_hattack1End	    128

#define FRAME_stand1start	    129
#define FRAME_stand1end		    147
#define FRAME_stand2start	    148
#define FRAME_stand2end		    168
#define FRAME_walkStart 	    169
#define FRAME_walkEnd   	    176
#define FRAME_runStart  	    177
#define FRAME_runEnd    	    183
#define FRAME_leapStart  	    184
#define FRAME_leapLoop      	187
#define FRAME_leapEndStart   	188
#define FRAME_leapEnd        	190
#define FRAME_attack1Start    191
#define FRAME_attack1End	    194
#define FRAME_attack2Start    195
#define FRAME_attack2End	    207
#define FRAME_pain1Start	    208
#define FRAME_pain1End		    211
#define FRAME_pain2Start	    212
#define FRAME_pain2End		    219
#define FRAME_die1Start		    220
#define FRAME_die1End		      231

#define MODEL_SCALE	1.000

#endif
