/* G:\quake2\baseq2\models/monsters/boss3/rider */

/* This file generated by ModelGen - Do NOT Modify */

#define FRAME_stand201        	0
#define FRAME_stand202        	1
#define FRAME_stand203        	2
#define FRAME_stand204        	3
#define FRAME_stand205        	4
#define FRAME_stand206        	5
#define FRAME_stand207        	6
#define FRAME_stand208        	7
#define FRAME_stand209        	8
#define FRAME_stand210        	9
#define FRAME_stand211        	10
#define FRAME_stand212        	11
#define FRAME_stand213        	12
#define FRAME_stand214        	13
#define FRAME_stand215        	14
#define FRAME_stand216        	15
#define FRAME_stand217        	16
#define FRAME_stand218        	17
#define FRAME_stand219        	18
#define FRAME_stand220        	19
#define FRAME_stand221        	20
#define FRAME_stand222        	21
#define FRAME_stand223        	22
#define FRAME_stand224        	23
#define FRAME_stand225        	24
#define FRAME_stand226        	25
#define FRAME_stand227        	26
#define FRAME_stand228        	27
#define FRAME_stand229        	28
#define FRAME_stand230        	29
#define FRAME_stand231        	30
#define FRAME_stand232        	31
#define FRAME_stand233        	32
#define FRAME_stand234        	33
#define FRAME_stand235        	34
#define FRAME_stand236        	35
#define FRAME_stand237        	36
#define FRAME_stand238        	37
#define FRAME_stand239        	38
#define FRAME_stand240        	39
#define FRAME_stand241        	40
#define FRAME_stand242        	41
#define FRAME_stand243        	42
#define FRAME_stand244        	43
#define FRAME_stand245        	44
#define FRAME_stand246        	45
#define FRAME_stand247        	46
#define FRAME_stand248        	47
#define FRAME_stand249        	48
#define FRAME_stand250        	49
#define FRAME_stand251        	50
#define FRAME_stand252        	51
#define FRAME_stand253        	52
#define FRAME_stand254        	53
#define FRAME_stand255        	54
#define FRAME_stand256        	55
#define FRAME_stand257        	56
#define FRAME_stand258        	57
#define FRAME_stand259        	58
#define FRAME_stand260        	59

#define MODEL_SCALE		1.000000
