// G:\quake2\baseq2\models/player_y

// This file generated by ModelGen - Do NOT Modify

#define FRAME_attak01         	0
#define FRAME_attak02         	1
#define FRAME_attak03         	2
#define FRAME_attak04         	3
#define FRAME_death101        	4
#define FRAME_death102        	5
#define FRAME_death103        	6
#define FRAME_death104        	7
#define FRAME_death105        	8
#define FRAME_death106        	9
#define FRAME_death107        	10
#define FRAME_death201        	11
#define FRAME_death202        	12
#define FRAME_death203        	13
#define FRAME_death204        	14
#define FRAME_death205        	15
#define FRAME_death206        	16
#define FRAME_death207        	17
#define FRAME_death208        	18
#define FRAME_death209        	19
#define FRAME_death210        	20
#define FRAME_death211        	21
#define FRAME_death212        	22
#define FRAME_death213        	23
#define FRAME_death301        	24
#define FRAME_death302        	25
#define FRAME_death303        	26
#define FRAME_death304        	27
#define FRAME_death305        	28
#define FRAME_death306        	29
#define FRAME_death307        	30
#define FRAME_death308        	31
#define FRAME_death309        	32
#define FRAME_death310        	33
#define FRAME_death311        	34
#define FRAME_death312        	35
#define FRAME_death313        	36
#define FRAME_death314        	37
#define FRAME_death315        	38
#define FRAME_flip01          	39
#define FRAME_flip02          	40
#define FRAME_flip03          	41
#define FRAME_flip04          	42
#define FRAME_flip05          	43
#define FRAME_flip06          	44
#define FRAME_flip07          	45
#define FRAME_flip08          	46
#define FRAME_flip09          	47
#define FRAME_flip10          	48
#define FRAME_flip11          	49
#define FRAME_flip12          	50
#define FRAME_flip13          	51
#define FRAME_flip14          	52
#define FRAME_grenad01        	53
#define FRAME_grenad02        	54
#define FRAME_grenad03        	55
#define FRAME_grenad04        	56
#define FRAME_grenad05        	57
#define FRAME_grenad06        	58
#define FRAME_grenad07        	59
#define FRAME_grenad08        	60
#define FRAME_grenad09        	61
#define FRAME_grenad10        	62
#define FRAME_grenad11        	63
#define FRAME_grenad12        	64
#define FRAME_grenad13        	65
#define FRAME_grenad14        	66
#define FRAME_grenad15        	67
#define FRAME_jump01          	68
#define FRAME_jump02          	69
#define FRAME_jump03          	70
#define FRAME_jump04          	71
#define FRAME_jump05          	72
#define FRAME_jump06          	73
#define FRAME_pain101         	74
#define FRAME_pain102         	75
#define FRAME_pain103         	76
#define FRAME_pain201         	77
#define FRAME_pain202         	78
#define FRAME_pain203         	79
#define FRAME_pain301         	80
#define FRAME_pain302         	81
#define FRAME_pain303         	82
#define FRAME_push01          	83
#define FRAME_push02          	84
#define FRAME_push03          	85
#define FRAME_push04          	86
#define FRAME_push05          	87
#define FRAME_push06          	88
#define FRAME_push07          	89
#define FRAME_push08          	90
#define FRAME_push09          	91
#define FRAME_run01           	92
#define FRAME_run02           	93
#define FRAME_run03           	94
#define FRAME_run04           	95
#define FRAME_run05           	96
#define FRAME_run06           	97
#define FRAME_run07           	98
#define FRAME_run08           	99
#define FRAME_run09           	100
#define FRAME_run10           	101
#define FRAME_run11           	102
#define FRAME_run12           	103
#define FRAME_runs01          	104
#define FRAME_runs02          	105
#define FRAME_runs03          	106
#define FRAME_runs04          	107
#define FRAME_runs05          	108
#define FRAME_runs06          	109
#define FRAME_runs07          	110
#define FRAME_runs08          	111
#define FRAME_runs09          	112
#define FRAME_runs10          	113
#define FRAME_runs11          	114
#define FRAME_runs12          	115
#define FRAME_salute01        	116
#define FRAME_salute02        	117
#define FRAME_salute03        	118
#define FRAME_salute04        	119
#define FRAME_salute05        	120
#define FRAME_salute06        	121
#define FRAME_salute07        	122
#define FRAME_salute08        	123
#define FRAME_salute09        	124
#define FRAME_salute10        	125
#define FRAME_salute11        	126
#define FRAME_salute12        	127
#define FRAME_stand101        	128
#define FRAME_stand102        	129
#define FRAME_stand103        	130
#define FRAME_stand104        	131
#define FRAME_stand105        	132
#define FRAME_stand106        	133
#define FRAME_stand107        	134
#define FRAME_stand108        	135
#define FRAME_stand109        	136
#define FRAME_stand110        	137
#define FRAME_stand111        	138
#define FRAME_stand112        	139
#define FRAME_stand113        	140
#define FRAME_stand114        	141
#define FRAME_stand115        	142
#define FRAME_stand116        	143
#define FRAME_stand117        	144
#define FRAME_stand118        	145
#define FRAME_stand119        	146
#define FRAME_stand120        	147
#define FRAME_stand121        	148
#define FRAME_stand122        	149
#define FRAME_stand123        	150
#define FRAME_stand124        	151
#define FRAME_stand125        	152
#define FRAME_stand126        	153
#define FRAME_stand127        	154
#define FRAME_stand128        	155
#define FRAME_stand129        	156
#define FRAME_stand130        	157
#define FRAME_stand131        	158
#define FRAME_stand132        	159
#define FRAME_stand133        	160
#define FRAME_stand134        	161
#define FRAME_stand135        	162
#define FRAME_stand136        	163
#define FRAME_stand137        	164
#define FRAME_stand138        	165
#define FRAME_stand139        	166
#define FRAME_stand140        	167
#define FRAME_stand201        	168
#define FRAME_stand202        	169
#define FRAME_stand203        	170
#define FRAME_stand204        	171
#define FRAME_stand205        	172
#define FRAME_stand206        	173
#define FRAME_stand207        	174
#define FRAME_stand208        	175
#define FRAME_stand209        	176
#define FRAME_stand210        	177
#define FRAME_stand211        	178
#define FRAME_stand212        	179
#define FRAME_stand213        	180
#define FRAME_stand214        	181
#define FRAME_stand215        	182
#define FRAME_stand216        	183
#define FRAME_stand217        	184
#define FRAME_stand218        	185
#define FRAME_stand219        	186
#define FRAME_stand220        	187
#define FRAME_stand221        	188
#define FRAME_stand222        	189
#define FRAME_stand223        	190
#define FRAME_swim01          	191
#define FRAME_swim02          	192
#define FRAME_swim03          	193
#define FRAME_swim04          	194
#define FRAME_swim05          	195
#define FRAME_swim06          	196
#define FRAME_swim07          	197
#define FRAME_swim08          	198
#define FRAME_swim09          	199
#define FRAME_swim10          	200
#define FRAME_swim11          	201
#define FRAME_swim12          	202
#define FRAME_sw_atk01        	203
#define FRAME_sw_atk02        	204
#define FRAME_sw_atk03        	205
#define FRAME_sw_atk04        	206
#define FRAME_sw_atk05        	207
#define FRAME_sw_atk06        	208
#define FRAME_sw_pan01        	209
#define FRAME_sw_pan02        	210
#define FRAME_sw_pan03        	211
#define FRAME_sw_pan04        	212
#define FRAME_sw_pan05        	213
#define FRAME_sw_std01        	214
#define FRAME_sw_std02        	215
#define FRAME_sw_std03        	216
#define FRAME_sw_std04        	217
#define FRAME_sw_std05        	218
#define FRAME_sw_std06        	219
#define FRAME_sw_std07        	220
#define FRAME_sw_std08        	221
#define FRAME_sw_std09        	222
#define FRAME_sw_std10        	223
#define FRAME_sw_std11        	224
#define FRAME_sw_std12        	225
#define FRAME_sw_std13        	226
#define FRAME_sw_std14        	227
#define FRAME_sw_std15        	228
#define FRAME_sw_std16        	229
#define FRAME_sw_std17        	230
#define FRAME_sw_std18        	231
#define FRAME_sw_std19        	232
#define FRAME_sw_std20        	233
#define FRAME_taunt01         	234
#define FRAME_taunt02         	235
#define FRAME_taunt03         	236
#define FRAME_taunt04         	237
#define FRAME_taunt05         	238
#define FRAME_taunt06         	239
#define FRAME_taunt07         	240
#define FRAME_taunt08         	241
#define FRAME_taunt09         	242
#define FRAME_taunt10         	243
#define FRAME_taunt11         	244
#define FRAME_taunt12         	245
#define FRAME_taunt13         	246
#define FRAME_taunt14         	247
#define FRAME_taunt15         	248
#define FRAME_taunt16         	249
#define FRAME_taunt17         	250
#define FRAME_walk01          	251
#define FRAME_walk02          	252
#define FRAME_walk03          	253
#define FRAME_walk04          	254
#define FRAME_walk05          	255
#define FRAME_walk06          	256
#define FRAME_walk07          	257
#define FRAME_walk08          	258
#define FRAME_walk09          	259
#define FRAME_walk10          	260
#define FRAME_walk11          	261
#define FRAME_wave01          	262
#define FRAME_wave02          	263
#define FRAME_wave03          	264
#define FRAME_wave04          	265
#define FRAME_wave05          	266
#define FRAME_wave06          	267
#define FRAME_wave07          	268
#define FRAME_wave08          	269
#define FRAME_wave09          	270
#define FRAME_wave10          	271
#define FRAME_wave11          	272
#define FRAME_wave12          	273
#define FRAME_wave13          	274
#define FRAME_wave14          	275
#define FRAME_wave15          	276
#define FRAME_wave16          	277
#define FRAME_wave17          	278
#define FRAME_wave18          	279
#define FRAME_wave19          	280
#define FRAME_wave20          	281
#define FRAME_wave21          	282
#define FRAME_bl_atk01        	283
#define FRAME_bl_atk02        	284
#define FRAME_bl_atk03        	285
#define FRAME_bl_atk04        	286
#define FRAME_bl_atk05        	287
#define FRAME_bl_atk06        	288
#define FRAME_bl_flp01        	289
#define FRAME_bl_flp02        	290
#define FRAME_bl_flp13        	291
#define FRAME_bl_flp14        	292
#define FRAME_bl_flp15        	293
#define FRAME_bl_jmp01        	294
#define FRAME_bl_jmp02        	295
#define FRAME_bl_jmp03        	296
#define FRAME_bl_jmp04        	297
#define FRAME_bl_jmp05        	298
#define FRAME_bl_jmp06        	299
#define FRAME_bl_pn101        	300
#define FRAME_bl_pn102        	301
#define FRAME_bl_pn103        	302
#define FRAME_bl_pn201        	303
#define FRAME_bl_pn202        	304
#define FRAME_bl_pn203        	305
#define FRAME_bl_pn301        	306
#define FRAME_bl_pn302        	307
#define FRAME_bl_pn303        	308
#define FRAME_bl_psh08        	309
#define FRAME_bl_psh09        	310
#define FRAME_bl_run01        	311
#define FRAME_bl_run02        	312
#define FRAME_bl_run03        	313
#define FRAME_bl_run04        	314
#define FRAME_bl_run05        	315
#define FRAME_bl_run06        	316
#define FRAME_bl_run07        	317
#define FRAME_bl_run08        	318
#define FRAME_bl_run09        	319
#define FRAME_bl_run10        	320
#define FRAME_bl_run11        	321
#define FRAME_bl_run12        	322
#define FRAME_bl_rns03        	323
#define FRAME_bl_rns04        	324
#define FRAME_bl_rns05        	325
#define FRAME_bl_rns06        	326
#define FRAME_bl_rns07        	327
#define FRAME_bl_rns08        	328
#define FRAME_bl_rns09        	329
#define FRAME_bl_sal10        	330
#define FRAME_bl_sal11        	331
#define FRAME_bl_sal12        	332
#define FRAME_bl_std01        	333
#define FRAME_bl_std02        	334
#define FRAME_bl_std03        	335
#define FRAME_bl_std04        	336
#define FRAME_bl_std05        	337
#define FRAME_bl_std06        	338
#define FRAME_bl_std07        	339
#define FRAME_bl_std08        	340
#define FRAME_bl_std09        	341
#define FRAME_bl_std10        	342
#define FRAME_bl_std11        	343
#define FRAME_bl_std12        	344
#define FRAME_bl_std13        	345
#define FRAME_bl_std14        	346
#define FRAME_bl_std15        	347
#define FRAME_bl_std16        	348
#define FRAME_bl_std17        	349
#define FRAME_bl_std18        	350
#define FRAME_bl_std19        	351
#define FRAME_bl_std20        	352
#define FRAME_bl_std21        	353
#define FRAME_bl_std22        	354
#define FRAME_bl_std23        	355
#define FRAME_bl_std24        	356
#define FRAME_bl_std25        	357
#define FRAME_bl_std26        	358
#define FRAME_bl_std27        	359
#define FRAME_bl_std28        	360
#define FRAME_bl_std29        	361
#define FRAME_bl_std30        	362
#define FRAME_bl_std31        	363
#define FRAME_bl_std32        	364
#define FRAME_bl_std33        	365
#define FRAME_bl_std34        	366
#define FRAME_bl_std35        	367
#define FRAME_bl_std36        	368
#define FRAME_bl_std37        	369
#define FRAME_bl_std38        	370
#define FRAME_bl_std39        	371
#define FRAME_bl_std40        	372
#define FRAME_bl_swm01        	373
#define FRAME_bl_swm02        	374
#define FRAME_bl_swm03        	375
#define FRAME_bl_swm04        	376
#define FRAME_bl_swm05        	377
#define FRAME_bl_swm06        	378
#define FRAME_bl_swm07        	379
#define FRAME_bl_swm08        	380
#define FRAME_bl_swm09        	381
#define FRAME_bl_swm10        	382
#define FRAME_bl_swm11        	383
#define FRAME_bl_swm12        	384
#define FRAME_bl_swk01        	385
#define FRAME_bl_swk02        	386
#define FRAME_bl_swk03        	387
#define FRAME_bl_swk04        	388
#define FRAME_bl_swk05        	389
#define FRAME_bl_swk06        	390
#define FRAME_bl_swp01        	391
#define FRAME_bl_swp02        	392
#define FRAME_bl_swp03        	393
#define FRAME_bl_swp04        	394
#define FRAME_bl_swp05        	395
#define FRAME_bl_sws01        	396
#define FRAME_bl_sws02        	397
#define FRAME_bl_sws03        	398
#define FRAME_bl_sws04        	399
#define FRAME_bl_sws05        	400
#define FRAME_bl_sws06        	401
#define FRAME_bl_sws07        	402
#define FRAME_bl_sws08        	403
#define FRAME_bl_sws09        	404
#define FRAME_bl_sws10        	405
#define FRAME_bl_sws11        	406
#define FRAME_bl_sws12        	407
#define FRAME_bl_sws13        	408
#define FRAME_bl_sws14        	409
#define FRAME_bl_tau14        	410
#define FRAME_bl_tau15        	411
#define FRAME_bl_tau16        	412
#define FRAME_bl_tau17        	413
#define FRAME_bl_wlk01        	414
#define FRAME_bl_wlk02        	415
#define FRAME_bl_wlk03        	416
#define FRAME_bl_wlk04        	417
#define FRAME_bl_wlk05        	418
#define FRAME_bl_wlk06        	419
#define FRAME_bl_wlk07        	420
#define FRAME_bl_wlk08        	421
#define FRAME_bl_wlk09        	422
#define FRAME_bl_wlk10        	423
#define FRAME_bl_wlk11        	424
#define FRAME_bl_wav19        	425
#define FRAME_bl_wav20        	426
#define FRAME_bl_wav21        	427
#define FRAME_cr_atk01        	428
#define FRAME_cr_atk02        	429
#define FRAME_cr_atk03        	430
#define FRAME_cr_atk04        	431
#define FRAME_cr_atk05        	432
#define FRAME_cr_atk06        	433
#define FRAME_cr_atk07        	434
#define FRAME_cr_atk08        	435
#define FRAME_cr_pan01        	436
#define FRAME_cr_pan02        	437
#define FRAME_cr_pan03        	438
#define FRAME_cr_pan04        	439
#define FRAME_cr_std01        	440
#define FRAME_cr_std02        	441
#define FRAME_cr_std03        	442
#define FRAME_cr_std04        	443
#define FRAME_cr_std05        	444
#define FRAME_cr_std06        	445
#define FRAME_cr_std07        	446
#define FRAME_cr_std08        	447
#define FRAME_cr_wlk01        	448
#define FRAME_cr_wlk02        	449
#define FRAME_cr_wlk03        	450
#define FRAME_cr_wlk04        	451
#define FRAME_cr_wlk05        	452
#define FRAME_cr_wlk06        	453
#define FRAME_cr_wlk07        	454
#define FRAME_crbl_a01        	455
#define FRAME_crbl_a02        	456
#define FRAME_crbl_a03        	457
#define FRAME_crbl_a04        	458
#define FRAME_crbl_a05        	459
#define FRAME_crbl_a06        	460
#define FRAME_crbl_a07        	461
#define FRAME_crbl_p01        	462
#define FRAME_crbl_p02        	463
#define FRAME_crbl_p03        	464
#define FRAME_crbl_p04        	465
#define FRAME_crbl_s01        	466
#define FRAME_crbl_s02        	467
#define FRAME_crbl_s03        	468
#define FRAME_crbl_s04        	469
#define FRAME_crbl_s05        	470
#define FRAME_crbl_s06        	471
#define FRAME_crbl_s07        	472
#define FRAME_crbl_s08        	473
#define FRAME_crbl_w01        	474
#define FRAME_crbl_w02        	475
#define FRAME_crbl_w03        	476
#define FRAME_crbl_w04        	477
#define FRAME_crbl_w05        	478
#define FRAME_crbl_w06        	479
#define FRAME_crbl_w07        	480

#define MODEL_SCALE		1.000000
