#ifndef Z_SENTIEN_H
#define Z_SENTIEN_H

/*
	Sentien constants
*/
#define FRAME_stand1start	0
#define FRAME_stand1end		28
#define FRAME_stand2start	29
#define FRAME_stand2end		48
#define FRAME_stand3start	49
#define FRAME_stand3end		79
#define FRAME_walkStartStart	80
#define FRAME_walkStartEnd	87
#define FRAME_walkLoopStart	88
#define FRAME_walkLoopEnd	103
#define FRAME_walkEndStart	104
#define FRAME_walkEndEnd	111

#define FRAME_blastPreStart	112
#define FRAME_blastPreEnd	115
#define FRAME_blastStart	116
#define FRAME_blastEnd		121
#define FRAME_blastPostStart	122
#define FRAME_blastPostEnd	125
#define FRAME_laserPreStart	126
#define FRAME_laserPreEnd	130
#define FRAME_laserStart	131
#define FRAME_laserEnd		141
#define FRAME_laserPostStart	142
#define FRAME_laserPostEnd	145
#define FRAME_slashPreStart   146
#define FRAME_slashPreEnd     150
#define FRAME_slashStart      151
#define FRAME_slashEnd        154
#define FRAME_slashPostStart  155
#define FRAME_slashPostEnd    167

#define FRAME_dodgeStart	168
#define FRAME_dodgeHold		173
#define FRAME_dodgeEnd		182
#define FRAME_pain1Start	183
#define FRAME_pain1End		186
#define FRAME_pain2Start	187
#define FRAME_pain2End		192
#define FRAME_pain3Start	193
#define FRAME_pain3End		213
#define FRAME_die1Start		214
#define FRAME_die1End		241
#define FRAME_die2Start		242
#define FRAME_die2End		270

#define MODEL_SCALE	1.000

#endif
